@@ -20,6 +20,8 @@
 #
 '''Main handler for overlays.'''
 
+from __future__ import unicode_literals
+
 __version__ = "$Id: overlay.py 273 2006-12-30 15:54:50Z wrobel $"
 
 #===============================================================================
@@ -32,20 +34,33 @@ import sys, os, os.path
 import xml
 import xml.etree.ElementTree as ET # Python 2.5
 
-from   layman.debug              import OUT
+#from   layman.debug              import OUT
 from   layman.utils              import indent
+from   layman.compatibility      import fileopen
 from   layman.overlays.overlay   import Overlay
 
+
+#py3.2+
+if sys.hexversion >= 0x30200f0:
+    _UNICODE = 'unicode'
+else:
+    _UNICODE = 'UTF-8'
+
+
 #===============================================================================
 #
 # Class UnknownOverlayException
 #
 #-------------------------------------------------------------------------------
+def UnknownOverlayMessage(ovl):
+    return 'Exception: Overlay "%s" does not exist.' % ovl
 
 class UnknownOverlayException(Exception):
     def __init__(self, repo_name):
-        message = 'Overlay "%s" does not exist.' % repo_name
-        super(UnknownOverlayException, self).__init__(message)
+        self.repo_name = repo_name
+
+    def __str__(self):
+        return UnknownOverlayMessage(self.repo_name)
 
 #===============================================================================
 #
@@ -71,116 +86,159 @@ class BrokenOverlayCatalog(ValueError):
 #
 #-------------------------------------------------------------------------------
 
-class DbBase:
+class DbBase(object):
     ''' Handle a list of overlays.'''
 
-    def __init__(self, paths, config, ignore = 0, quiet = False, ignore_init_read_errors=False):
+    def __init__(self, config, paths=None, ignore = 0,
+        ignore_init_read_errors=False
+        ):
 
         self.config = config
-        self.quiet = quiet
         self.paths = paths
         self.ignore = ignore
+        self.output = config['output']
+        self.ignore_init_read_errors = ignore_init_read_errors
 
         self.overlays = {}
 
-        OUT.debug('Initializing overlay list handler', 8)
+        self.output.debug('Initializing overlay list handler', 8)
 
+        path_found = False
         for path in self.paths:
             if not os.path.exists(path):
                 continue
 
-            try:
-                self.read_file(path)
-            except Exception, error:
-                if not ignore_init_read_errors: raise error
+            self.read_file(path)
+            path_found = True
+
+        if not path_found:
+            self.output.warn("Warning: an installed db file was not found at: %s"
+                % str(self.paths))
+
 
     def __eq__(self, other):
-        for key in set(self.overlays.keys() + other.overlays.keys()):
+        for key in set(self.overlays.keys()) | set(other.overlays.keys()):
             if self.overlays[key] != other.overlays[key]:
                 return False
         return True
 
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
+
     def read_file(self, path):
         '''Read the overlay definition file.'''
 
         try:
-            document = open(path, 'r').read()
+            with fileopen(path, 'r') as df:
+                document = df.read()
 
-        except Exception, error:
-            raise IOError('Failed to read the overlay list at ("'
-                          + path + '")!\nError was:\n' + str(error))
+        except Exception as error:
+            if not self.ignore_init_read_errors:
+                self.output.error('Failed to read the overlay list at ("'
+                    + path + '")')
+                raise error
 
         self.read(document, origin=path)
 
+
     def _broken_catalog_hint(self):
         this_function_name = sys._getframe().f_code.co_name
         raise NotImplementedError('Method "%s.%s" not implemented' % \
                 (self.__class__.__name__, this_function_name))
 
+
     def read(self, text, origin):
         '''
         Read an xml list of overlays (adding to and potentially overwriting existing entries)
 
         >>> here = os.path.dirname(os.path.realpath(__file__))
-        >>> config = {'svn_command': '/usr/bin/svn', 'rsync_command':'/usr/bin/rsync'}
-        >>> a = DbBase([here + '/tests/testfiles/global-overlays.xml', ], config)
+        >>> from layman.output import Message
+        >>> output = Message()
+        >>> config = {'output': output, 'svn_command': '/usr/bin/svn', 'rsync_command':'/usr/bin/rsync'}
+        >>> a = DbBase(config, [here + '/tests/testfiles/global-overlays.xml', ])
         >>> a.overlays.keys()
-        [u'wrobel', u'wrobel-stable']
+        ['wrobel', 'wrobel-stable']
 
         >>> list(a.overlays['wrobel-stable'].source_uris())
-        [u'rsync://gunnarwrobel.de/wrobel-stable']
+        ['rsync://gunnarwrobel.de/wrobel-stable']
         '''
         try:
             document = ET.fromstring(text)
-        except xml.parsers.expat.ExpatError, error:
+        except xml.parsers.expat.ExpatError as error:
             raise BrokenOverlayCatalog(origin, error, self._broken_catalog_hint())
 
         overlays = document.findall('overlay') + \
                 document.findall('repo')
 
         for overlay in overlays:
-            OUT.debug('Parsing overlay entry', 8)
-            try:
-                ovl = Overlay(overlay, self.config, self.ignore, self.quiet)
-            except Exception, error:
-                OUT.warn(str(error), 3)
-            else:
-                self.overlays[ovl.name] = ovl
+            self.output.debug('Parsing overlay: %s' % overlay, 9)
+            ovl = Overlay(config=self.config, xml=overlay,
+                    ignore=self.ignore)
+            self.overlays[ovl.name] = ovl
+        return
+
+
+    def add_new(self, xml=None, origin=None, from_dict=None):
+        '''Reads xml text and dictionary definitions and adds
+        them to the db.
+        '''
+        if xml is not None:
+            self.read(xml, origin)
+        if from_dict is not None:
+            self.output.info("DbBase: add_new() from_dict")
+            if isinstance(from_dict, dict):
+                from_dict = [from_dict]
+            self._add_from_dict(from_dict)
+
+        return
+
+
+    def _add_from_dict(self, overlays=None):
+        """Add a new overlay from a list of dictionary values
+        """
+        self.output.info("DbBase: add_from_dict()")
+        for overlay in overlays:
+            self.output.debug('Parsing overlay entry', 8)
+            ovl = Overlay(self.config, ovl_dict=overlay,
+                    ignore=self.ignore)
+            self.overlays[ovl.name] = ovl
+        return
 
 
     def write(self, path):
         '''
         Write the list of overlays to a file.
 
-        >>> write = os.tmpnam()
+        >>> import tempfile
+        >>> tmpdir = tempfile.mkdtemp(prefix="laymantmp_")
+        >>> write = os.path.join(tmpdir, 'test.xml')
         >>> here = os.path.dirname(os.path.realpath(__file__))
-        >>> config = {'svn_command': '/usr/bin/svn', 'rsync_command':'/usr/bin/rsync'}
-        >>> a = DbBase([here + '/tests/testfiles/global-overlays.xml', ], config)
-        >>> b = DbBase([write,], dict())
+        >>> from layman.config import BareConfig
+        >>> config = BareConfig()
+        >>> a = DbBase(config, [here + '/tests/testfiles/global-overlays.xml', ])
+        >>> from layman.output import Message
+        >>> b = DbBase({"output": Message() }, [write,])
         >>> b.overlays['wrobel-stable'] = a.overlays['wrobel-stable']
         >>> b.write(write)
-        >>> c = DbBase([write,], dict())
+        >>> c = DbBase({"output": Message() }, [write,])
         >>> c.overlays.keys()
-        [u'wrobel-stable']
+        ['wrobel-stable']
 
         >>> os.unlink(write)
+        >>> os.rmdir(tmpdir)
         '''
 
-        tree = ET.Element('repositories', version="1.0")
+        tree = ET.Element('repositories', version="1.0", encoding=_UNICODE)
         tree[:] = [e.to_xml() for e in self.overlays.values()]
         indent(tree)
         tree = ET.ElementTree(tree)
         try:
-            f = open(path, 'w')
-            f.write("""\
-<?xml version="1.0" encoding="UTF-8"?>
-""")
-            tree.write(f, encoding='utf-8')
-            f.close()
-        except Exception, error:
+            with fileopen(path, 'w') as f:
+                 tree.write(f, encoding=_UNICODE)
+
+        except Exception as error:
             raise Exception('Failed to write to local overlays file: '
                             + path + '\nError was:\n' + str(error))
 
@@ -189,23 +247,32 @@ class DbBase:
         Select an overlay from the list.
 
         >>> here = os.path.dirname(os.path.realpath(__file__))
-        >>> config = {'svn_command': '/usr/bin/svn', 'rsync_command':'/usr/bin/rsync'}
-        >>> a = DbBase([here + '/tests/testfiles/global-overlays.xml', ], config)
+        >>> from layman.output import Message
+        >>> output = Message()
+        >>> config = {'output': output, 'svn_command': '/usr/bin/svn', 'rsync_command':'/usr/bin/rsync'}
+        >>> a = DbBase(config, [here + '/tests/testfiles/global-overlays.xml', ])
         >>> list(a.select('wrobel-stable').source_uris())
-        [u'rsync://gunnarwrobel.de/wrobel-stable']
+        ['rsync://gunnarwrobel.de/wrobel-stable']
         '''
+        self.output.debug("DbBase.select(), overlay = %s" % overlay, 5)
         if not overlay in self.overlays.keys():
+            self.output.debug("DbBase.select(), unknown overlay = %s"
+                % overlay, 4)
+            self.output.debug("DbBase.select(), known overlays = %s"
+                % ', '.join(self.overlays.keys()), 4)
             raise UnknownOverlayException(overlay)
         return self.overlays[overlay]
 
-    def list(self, verbose = False, width = 0):
+    def list(self, repos=None, verbose = False, width = 0):
         '''
         List all overlays.
 
         >>> here = os.path.dirname(os.path.realpath(__file__))
-        >>> config = {'svn_command': '/usr/bin/svn', 'rsync_command':'/usr/bin/rsync'}
-        >>> a = DbBase([here + '/tests/testfiles/global-overlays.xml', ], config)
-        >>> for i in a.list(True):
+        >>> from layman.output import Message
+        >>> output = Message()
+        >>> config = {'output': output, 'svn_command': '/usr/bin/svn', 'rsync_command':'/usr/bin/rsync'}
+        >>> a = DbBase(config, [here + '/tests/testfiles/global-overlays.xml', ])
+        >>> for i in a.list(verbose=True):
         ...     print i[0]
         wrobel
         ~~~~~~
@@ -228,26 +295,35 @@ class DbBase:
           A collection of ebuilds from Gunnar Wrobel [wrobel@gentoo.org].
         <BLANKLINE>
 
-        >>> for i in a.list(False, 80):
+        >>> for i in a.list(verbose=False, width=80):
         ...     print i[0]
         wrobel                    [Subversion] (https://o.g.o/svn/dev/wrobel         )
         wrobel-stable             [Rsync     ] (rsync://gunnarwrobel.de/wrobel-stable)
         '''
         result = []
 
-        for _, overlay in self.overlays.items():
+        selection = [overlay for (a, overlay) in self.overlays.items()]
+        if repos is not None:
+            selection = [overlay for overlay in selection if overlay.name in repos]
 
+        for overlay in selection:
             if verbose:
-                result.append((str(overlay), overlay.is_supported(),
+                result.append((overlay.get_infostr(), overlay.is_supported(),
                                overlay.is_official()))
             else:
                 result.append((overlay.short_list(width), overlay.is_supported(),
                                overlay.is_official()))
 
-        result = sorted(result, key=lambda (summary, supported, official): summary.lower())
+        result = sorted(result, key=lambda summary_supported_official: summary_supported_official[0].lower())
 
         return result
 
+    def list_ids(self):
+        """returns a list of the overlay names
+        """
+        return sorted(self.overlays)
+
+
 #===============================================================================
 #
 # Testing
