@@ -18,46 +18,37 @@ import sys, inspect, types
 
 from   optparse      import OptionGroup
 
-#################################################################################
-##
-## Color codes (taken from portage)
-##
-#################################################################################
+from   layman.constants  import (codes, DEBUG_LEVEL, DEBUG_VERBOSITY,
+    INFO_LEVEL, WARN_LEVEL, OFF)
 
-esc_seq = '\x1b['
+from output import Message
 
-codes = {}
-codes['reset']     = esc_seq + '39;49;00m'
-codes['red']       = esc_seq + '31;01m'
-codes['green']     = esc_seq + '32;01m'
-codes['yellow']    = esc_seq + '33;01m'
-codes['turquoise'] = esc_seq + '36;01m'
 
-#################################################################################
-##
-## Message Class
-##
-#################################################################################
-
-class Message:
+class DebugMessage(Message):
+    """fully Debug enabled subclass of output.py's Message
+    """
     #FIXME: Think about some simple doctests before you modify this class the
     #       next time.
 
     def __init__(self, module = '',
+                 out = sys.stdout,
                  err = sys.stderr,
                  dbg = sys.stderr,
-                 debugging_level = 4,
-                 debugging_verbosity = 2,
-                 info_level = 4,
-                 warn_level = 4,
+                 debug_level = DEBUG_LEVEL,
+                 debug_verbosity = DEBUG_VERBOSITY,
+                 info_level = INFO_LEVEL,
+                 warn_level = WARN_LEVEL,
                  col = True,
                  mth = None,
                  obj = None,
                  var = None):
+
         if mth == None: mth = ['*']
         if obj == None: obj = ['*']
         if var == None: var = ['*']
 
+        Message.__init__(self)
+
         # A description of the module that is being debugged
         self.debug_env = module
 
@@ -65,21 +56,24 @@ class Message:
         self.debug_out = dbg
 
         # Where should the error output go? This can also be a file
-        self.error_out = err
+        #self.error_out = err
+
+        # Where should the normal output go? This can also be a file
+        #self.std_out = out
 
         # The higher the level the more information you will get
-        self.warn_lev = warn_level
+        #self.warn_lev = warn_level
 
         # The higher the level the more information you will get
-        self.info_lev = info_level
+        #self.info_lev = info_level
 
         # The highest level of debugging messages acceptable for output
         # The higher the level the more output you will get
-        self.debug_lev = debugging_level
+        #self.debug_lev = debugging_level
 
         # The debugging output can range from very verbose (3) to
         # very compressed (1)
-        self.debug_vrb = debugging_verbosity
+        self.debug_vrb = debug_verbosity
 
         # Which methods should actually be debugged?
         # Use '*' to indicate 'All methods'
@@ -97,9 +91,9 @@ class Message:
         self.show_class_variables = False
 
         # Should the output be colored?
-        self.use_color = col
+        #self.use_color = col
 
-        self.has_error = False
+        #self.has_error = False
 
 
     ############################################################################
@@ -107,6 +101,8 @@ class Message:
 
     def cli_opts(self, parser):
 
+        #print "Parsing debug opts"
+
         group = OptionGroup(parser,
                             '<Debugging options>',
                             'Control the debugging features of '
@@ -175,9 +171,9 @@ class Message:
 
         if (options.__dict__.has_key('debug')
             and options.__dict__['debug']):
-            self.debug_on()
+            self.set_debug_level(DEBUG_LEVEL)
         else:
-            self.debug_off()
+            self.set_debug_level(OFF)
             return
 
         if (options.__dict__.has_key('debug_class_vars')
@@ -188,9 +184,9 @@ class Message:
 
         if (options.__dict__.has_key('debug_nocolor')
             and options.__dict__['debug_nocolor']):
-            self.color_off()
+            self.set_colorize(False)
         else:
-            self.color_on()
+            self.set_colorize(True)
 
         if (options.__dict__.has_key('debug_level') and
             options.__dict__['debug_level']):
@@ -247,46 +243,10 @@ class Message:
         if variables:
             self.debug_var = variables
 
-    def maybe_color (self, col, text):
-        if self.use_color:
-            return codes[col] + text + codes['reset']
-        return text
 
-    def set_info_level(self, info_level = 4):
-        self.info_lev = info_level
-
-    def info_off(self):
-        self.set_info_level(0)
-
-    def info_on(self, info_level = 4):
-        self.set_info_level(info_level)
-
-    def set_warn_level(self, warn_level = 4):
-        self.warn_lev = warn_level
-
-    def warn_off(self):
-        self.set_warn_level(0)
-
-    def warn_on(self, warn_level = 4):
-        self.set_warn_level(warn_level)
-
-    def set_debug_level(self, debugging_level = 4):
-        self.debug_lev = debugging_level
-
-    def set_debug_verbosity(self, debugging_verbosity = 2):
+    def set_debug_verbosity(self, debugging_verbosity = DEBUG_VERBOSITY):
         self.debug_vrb = debugging_verbosity
 
-    def debug_off(self):
-        self.set_debug_level(0)
-
-    def debug_on(self):
-        self.set_debug_level()
-
-    def color_off(self):
-        self.use_color = False
-
-    def color_on(self):
-        self.use_color = True
 
     def class_variables_off(self):
         self.show_class_variables = False
@@ -297,84 +257,7 @@ class Message:
     #############################################################################
     ## Output Functions
 
-    def notice (self, note):
-        print note
-
-    def info (self, info, level = 4):
-
-        if type(info) not in types.StringTypes:
-            info = str(info)
-
-        if level > self.info_lev:
-            return
-
-        for i in info.split('\n'):
-            print self.maybe_color('green', '* ') + i
-
-    def status (self, message, status, info = 'ignored'):
-
-        if type(message) not in types.StringTypes:
-            message = str(message)
-
-        lines = message.split('\n')
-
-        if not lines:
-            return
-
-        for i in lines[0:-1]:
-            print self.maybe_color('green', '* ') + i
-
-        i = lines[-1]
-
-        if len(i) > 58:
-            i = i[0:57]
-
-        if status == 1:
-            result = '[' + self.maybe_color('green', 'ok') + ']'
-        elif status == 0:
-            result = '[' + self.maybe_color('red', 'failed') + ']'
-        else:
-            result = '[' + self.maybe_color('yellow', info) + ']'
-
-        print self.maybe_color('green', '* ') + i + ' ' + '.' * (58 - len(i))  \
-              + ' ' + result
-
-    def warn (self, warn, level = 4):
-
-        if type(warn) not in types.StringTypes:
-            warn = str(warn)
-
-        if level > self.warn_lev:
-            return
-
-        for i in warn.split('\n'):
-            print self.maybe_color('yellow', '* ') + i
-
-    def error (self, error):
-
-        if type(error) not in types.StringTypes:
-            error = str(error)
-
-        for i in error.split('\n'):
-            # NOTE: Forced flushing ensures that stdout and stderr
-            # stay in nice order.  This is a workaround for calls like
-            # "layman -L |& less".
-            sys.stdout.flush()
-            print >> self.error_out, self.maybe_color('red', '* ') + i
-            self.error_out.flush()
-        self.has_error = True
-
-    def die (self, error):
-
-        if type(error) not in types.StringTypes:
-            error = str(error)
-
-        for i in error.split('\n'):
-            self.error(self.maybe_color('red', 'Fatal error: ') + i)
-        self.error(self.maybe_color('red', 'Fatal error(s) - aborting'))
-        sys.exit(1)
-
-    def debug (self, message, level = 4):
+    def debug (self, message, level = DEBUG_LEVEL):
         '''
         This is a generic debugging method.
         '''
@@ -448,7 +331,7 @@ class Message:
                     ## Remove printed characters from output
                     x = x[60:]
             ## Print final line
-            print >> self.debug_out, ls + '// ' + x 
+            print >> self.debug_out, ls + '// ' + x
 
         if self.debug_vrb == 1:
             # Top line indicates class and method
