@@ -16,6 +16,8 @@
 #
 ''' Cvs overlay support.'''
 
+from __future__ import unicode_literals
+
 __version__ = "$Id$"
 
 #===============================================================================
@@ -25,6 +27,7 @@ __version__ = "$Id$"
 #-------------------------------------------------------------------------------
 
 import xml.etree.ElementTree as ET # Python 2.5
+import re
 
 from   layman.utils             import path
 from   layman.overlays.source   import OverlaySource, require_supported
@@ -44,25 +47,9 @@ class CvsOverlay(OverlaySource):
     def __init__(self, parent, config, _location, ignore = 0):
 
         super(CvsOverlay, self).__init__(parent, config, _location, ignore)
-        self.subpath = None
+        self.branch = self.parent.branch
 
 
-    def __eq__(self, other):
-        res = super(CvsOverlay, self).__eq__(other) \
-            and self.subpath == other.subpath
-        return res
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    # overrider
-    def to_xml_hook(self, repo_elem):
-        if self.subpath:
-            _subpath = ET.Element('subpath')
-            _subpath.text = self.subpath
-            repo_elem.append(_subpath)
-            del _subpath
-
     def add(self, base):
         '''Add overlay.'''
 
@@ -81,13 +68,49 @@ class CvsOverlay(OverlaySource):
         if len(cfg_opts):
             args.append(cfg_opts)
         args.append(self.parent.name)
-        args.append(self.subpath)
+        args.append(self.branch)
 
         return self.postsync(
             self.run_command(self.command(), args, cwd=base,
                 env=dict(CVSROOT=self.src), cmd=self.type),
             cwd=target)
 
+    def update(self, base, src):
+        '''
+        Updates overlay src-url.
+        
+        @params base: base location where all overlays are installed.
+        @params src: source URL.
+        '''
+        
+        if not self.supported():
+            return 1
+
+        target = path([base, self.parent.name])
+        
+        # First echo the new repository to CVS/Root
+        args = [src, '>', '/CVS/Root']
+        result = self.run_command('echo', args, cmd='echo', cwd=target)
+
+        if result == 0:
+            location = src.split(':')[3]
+            old_location = self.src.split(':/')[3]
+    
+            # Check if the repository location needs to be updated too.
+            if not location == old_location:
+                location = re.sub('/', '\/', location)
+                old_location = re.sub('/', '\/', old_location)
+                
+                expression = 's/' + old_location + '/' + location + '/'
+
+                # sed -i 's/<old_location>/<new_location>/ <target>/CVS/Repository
+                args = ['-i', expression, '/CVS/Repository']
+
+                return self.run_command('sed', args, cmd='sed', cwd=target)
+
+        return result        
+        
+
     def sync(self, base):
         '''Sync overlay.'''
 
