@@ -17,6 +17,8 @@
 #
 ''' Darcs overlay support.'''
 
+from __future__ import unicode_literals
+
 __version__ = "$Id: darcs.py 236 2006-09-05 20:39:37Z wrobel $"
 
 #===============================================================================
@@ -40,30 +42,59 @@ class DarcsOverlay(OverlaySource):
     type = 'Darcs'
     type_key = 'darcs'
 
-    def __init__(self, parent, xml, config, _location, ignore = 0, quiet = False):
+    def __init__(self, parent, config, _location, ignore = 0):
 
-        super(DarcsOverlay, self).__init__(parent, xml, config, _location, ignore, quiet)
+        super(DarcsOverlay, self).__init__(parent, config,
+            _location, ignore)
+        self.branch = None
 
-    def add(self, base, quiet = False):
+    def add(self, base):
         '''Add overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
+
+        cfg_opts = self.config["darcs_addopts"]
+        target = path([base, self.parent.name])
+
+        if self.src.endswith("/"):
+            src = self.src
+        else:
+            src = self.src + '/'
 
         # darcs get --partial SOURCE TARGET
-        args = ['get', '--partial', self.src + '/', path([base, self.parent.name])]
-        return self.run_command(*args)
+        if len(cfg_opts):
+            args = ['get', '--partial', cfg_opts,
+                src, target]
+        else:
+            args = ['get', '--partial',
+                src, target]
 
-    def sync(self, base, quiet = False):
+        return self.postsync(
+            self.run_command(self.command(), args, cmd=self.type),
+            cwd=target)
+
+    def sync(self, base):
         '''Sync overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
+
+        cfg_opts = self.config["darcs_addopts"]
+        target = path([base, self.parent.name])
 
         # darcs pull --all SOURCE
-        args = ['pull', '--all', self.src]
-        return self.run_command(*args, cwd=path([base, self.parent.name]))
+        if len(cfg_opts):
+            args = ['pull', '--all', cfg_opts, self.src]
+        else:
+            args = ['pull', '--all', self.src]
+        return self.postsync(
+            self.run_command(self.command(), args, cwd=target, cmd=self.type),
+            cwd=target)
 
     def supported(self):
         '''Overlay type supported?'''
 
-        return require_supported([(self.command(),  'darcs',
-                                         'dev-vcs/darcs'),])
+        return require_supported(
+            [(self.command(),  'darcs', 'dev-vcs/darcs'),],
+            self.output.warn)
