@@ -16,6 +16,8 @@
 #
 ''' Rsync overlay support.'''
 
+from __future__ import unicode_literals
+
 __version__ = "$Id: rsync.py 236 2006-09-05 20:39:37Z wrobel $"
 
 #===============================================================================
@@ -44,7 +46,7 @@ class RsyncOverlay(OverlaySource):
 
         super(RsyncOverlay, self).__init__(parent, config,
             _location, ignore)
-        self.subpath = None
+        self.branch = None
 
     def add(self, base):
         '''Add overlay.'''
