@@ -16,6 +16,8 @@
 #
 ''' Tar overlay support.'''
 
+from __future__ import unicode_literals
+
 __version__ = "$Id: tar.py 310 2007-04-09 16:30:40Z wrobel $"
 
 #===============================================================================
@@ -24,12 +26,21 @@ __version__ = "$Id: tar.py 310 2007-04-09 16:30:40Z wrobel $"
 #
 #-------------------------------------------------------------------------------
 
-import os, os.path, sys, urllib2, shutil, tempfile
+import os
+import os.path
+import sys
+import shutil
+import tempfile
+
 import xml.etree.ElementTree as ET # Python 2.5
 
-from   layman.utils             import path, ensure_unicode
-from   layman.debug             import OUT
+from   layman.compatibility     import fileopen
 from   layman.overlays.source   import OverlaySource, require_supported
+from   layman.utils             import path
+from   layman.version           import VERSION
+from   sslfetch.connections     import Connector
+
+USERAGENT = "Layman" + VERSION
 
 #===============================================================================
 #
@@ -40,7 +51,7 @@ from   layman.overlays.source   import OverlaySource, require_supported
 class TarOverlay(OverlaySource):
     ''' Handles tar overlays.
 
-    >>> from   layman.debug             import OUT
+    >>> from   layman.output import Message
     >>> import xml.etree.ElementTree as ET # Python 2.5
     >>> repo = ET.Element('repo')
     >>> repo_name = ET.Element('name')
@@ -54,18 +65,21 @@ class TarOverlay(OverlaySource):
     >>> source = ET.Element('source', type='tar')
     >>> here = os.path.dirname(os.path.realpath(__file__))
     >>> source.text = 'file://' + here + '/../tests/testfiles/layman-test.tar.bz2'
-    >>> subpath = ET.Element('subpath')
-    >>> subpath.text = 'layman-test'
-    >>> repo[:] = [repo_name, desc, owner, source, subpath]
-    >>> config = {'tar_command':'/bin/tar'}
-    >>> testdir = os.tmpnam()
-    >>> os.mkdir(testdir)
+    >>> branch = ET.Element('branch')
+    >>> branch.text = 'layman-test'
+    >>> repo[:] = [repo_name, desc, owner, source, branch]
+    >>> from layman.config import BareConfig
+    >>> config = BareConfig()
+    >>> import tempfile
+    >>> testdir = tempfile.mkdtemp(prefix="laymantmp_")
     >>> from layman.overlays.overlay import Overlay
-    >>> a = Overlay(repo, config, quiet=False)
-    >>> OUT.color_off()
-    >>> a.add(testdir) #doctest: +ELLIPSIS
-    * Running... # /bin/tar -v -x -f...
-    >>> sorted(os.listdir(testdir + '/dummy'))
+    >>> a = Overlay(config, repo)
+    >>> config['output'].set_colorize(False)
+    >>> a.add(testdir)
+    0
+    >>> os.listdir(testdir + '/dummy/')
+    ['layman-test']
+    >>> sorted(os.listdir(testdir + '/dummy/layman-test/'))
     ['app-admin', 'app-portage']
     >>> shutil.rmtree(testdir)
     '''
@@ -73,36 +87,19 @@ class TarOverlay(OverlaySource):
     type = 'Tar'
     type_key = 'tar'
 
-    def __init__(self, parent, xml, config, _location, ignore = 0, quiet = False):
+    def __init__(self, parent, config, _location, ignore = 0):
 
-        super(TarOverlay, self).__init__(parent, xml, config, _location, ignore, quiet)
+        super(TarOverlay, self).__init__(parent,
+            config, _location, ignore)
 
-        _subpath = xml.find('subpath')
-        if _subpath != None:
-            self.subpath = ensure_unicode(_subpath.text.strip())
-        elif 'subpath' in xml.attrib:
-            self.subpath = ensure_unicode(xml.attrib['subpath'])
-        else:
-            self.subpath = ''
+        self.output = config['output']
+        self.proxies = config.proxies
+        self.branch = self.parent.branch
 
-    def __eq__(self, other):
-        res = super(TarOverlay, self).__eq__(other) \
-            and self.subpath == other.subpath
-        return res
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    # overrider
-    def to_xml_hook(self, repo_elem):
-        if self.subpath:
-            _subpath = ET.Element('subpath')
-            _subpath.text = self.subpath
-            repo_elem.append(_subpath)
-            del _subpath
 
     def _extract(self, base, tar_url, dest_dir):
         ext = '.tar.noidea'
+        clean_tar = self.config['clean_tar']
         for i in [('tar.%s' % e) for e in ('bz2', 'gz', 'lzma', 'xz', 'Z')] \
                 + ['tgz', 'tbz', 'taz', 'tlz', 'txz']:
             candidate_ext = '.%s' % i
@@ -110,52 +107,64 @@ class TarOverlay(OverlaySource):
                 ext = candidate_ext
                 break
 
-        try:
-            tar = urllib2.urlopen(tar_url).read()
-        except Exception, error:
-            raise Exception('Failed to fetch the tar package from: '
-                            + self.src + '\nError was:' + str(error))
+        if 'file://' not in tar_url:
+            # setup the ssl-fetch output map
+            connector_output = {
+                'info':  self.output.debug,
+                'error': self.output.error,
+                'kwargs-info': {'level': 2},
+                'kwargs-error':{'level': None},
+            }
 
-        pkg = path([base, self.parent.name + ext])
+            fetcher = Connector(connector_output, self.proxies, USERAGENT)
 
-        try:
-            out_file = open(pkg, 'w')
-            out_file.write(tar)
-            out_file.close()
-        except Exception, error:
-            raise Exception('Failed to store tar package in '
-                            + pkg + '\nError was:' + str(error))
+            success, tar, timestamp = fetcher.fetch_content(tar_url)
+
+            pkg = path([base, self.parent.name + ext])
+
+            try:
+                with fileopen(pkg, 'w+b') as out_file:
+                    out_file.write(tar)
+
+            except Exception as error:
+                raise Exception('Failed to store tar package in '
+                                + pkg + '\nError was:' + str(error))
+        else:
+            clean_tar = False
+            pkg = tar_url.replace('file://', '')
 
         # tar -v -x -f SOURCE -C TARGET
         args = ['-v', '-x', '-f', pkg, '-C', dest_dir]
-        result = self.run_command(*args)
+        result = self.run_command(self.command(), args, cmd=self.type)
 
-        os.unlink(pkg)
+        if clean_tar:
+            os.unlink(pkg)
         return result
 
-    def _add_unchecked(self, base, quiet):
+    def _add_unchecked(self, base):
         def try_to_wipe(folder):
             if not os.path.exists(folder):
                 return
 
             try:
-                OUT.info('Deleting directory "%s"' % folder, 2)
+                self.output.info('Deleting directory "%s"' % folder, 2)
                 shutil.rmtree(folder)
-            except Exception, error:
+            except Exception as error:
                 raise Exception('Failed to remove unnecessary tar structure "'
                                 + folder + '"\nError was:' + str(error))
 
         final_path = path([base, self.parent.name])
         temp_path = tempfile.mkdtemp(dir=base)
         try:
-            result = self._extract(base=base, tar_url=self.src, dest_dir=temp_path)
-        except Exception, error:
+            result = self._extract(base=base, tar_url=self.src,
+                dest_dir=temp_path)
+        except Exception as error:
             try_to_wipe(temp_path)
             raise error
 
         if result == 0:
-            if self.subpath:
-                source = temp_path + '/' + self.subpath
+            if self.branch:
+                source = temp_path + '/' + self.branch
             else:
                 source = temp_path
 
@@ -165,40 +174,52 @@ class TarOverlay(OverlaySource):
 
                 try:
                     os.rename(source, final_path)
-                except Exception, error:
+                except Exception as error:
                     raise Exception('Failed to rename tar subdirectory ' +
-                                    source + ' to ' + final_path + '\nError was:'
-                                    + str(error))
-                os.chmod(final_path, 0755)
+                                    source + ' to ' + final_path +
+                                    '\nError was:' + str(error))
+                os.chmod(final_path, 0o755)
             else:
-                raise Exception('Given subpath "' + source + '" does not exist '
-                                ' in the tar package!')
+                raise Exception('The given path (branch setting in the xml)\n' + \
+                    '"%(source)s" does not exist in the tar package!' % ({'source': source}))
 
         try_to_wipe(temp_path)
         return result
 
-    def add(self, base, quiet = False):
+    def add(self, base):
         '''Add overlay.'''
 
-        self.supported()
+        if not self.supported():
+            return 1
 
-        final_path = path([base, self.parent.name])
+        target = path([base, self.parent.name])
 
-        if os.path.exists(final_path):
-            raise Exception('Directory ' + final_path + ' already exists. Will not ov'
-                            'erwrite its contents!')
+        if os.path.exists(target):
+            raise Exception('Directory ' + target + ' already exists.' +\
+                ' Will not overwrite its contents!')
 
-        return self._add_unchecked(base, quiet)
+        return self.postsync(
+            self._add_unchecked(base),
+            cwd=target)
 
-    def sync(self, base, quiet = False):
+    def sync(self, base):
         '''Sync overlay.'''
-        self.supported()
-        self._add_unchecked(base, quiet)
+
+        if not self.supported():
+            return 1
+
+        target = path([base, self.parent.name])
+
+        return self.postsync(
+            self._add_unchecked(base),
+            cwd=target)
 
     def supported(self):
         '''Overlay type supported?'''
 
-        return require_supported([(self.command(),  'tar', 'app-arch/tar'), ])
+        return require_supported(
+            [(self.command(),  'tar', 'app-arch/tar'), ],
+            self.output.warn)
 
 if __name__ == '__main__':
     import doctest
