@@ -15,12 +15,18 @@
 #
 '''Runs external (non-doctest) test cases.'''
 
-import unittest
 import os
-import tempfile
 import shutil
-import urllib
+import tempfile
+import unittest
+#Py3
+try:
+    import urllib.request as urllib
+except ImportError:
+    import urllib
 from layman.dbbase import DbBase
+from layman.output import Message
+from layman.config import BareConfig
 from warnings import filterwarnings, resetwarnings
 
 HERE = os.path.dirname(os.path.realpath(__file__))
@@ -28,13 +34,13 @@ HERE = os.path.dirname(os.path.realpath(__file__))
 
 class Unicode(unittest.TestCase):
     def _overlays_bug(self, number):
-        config = {}
+        config = BareConfig()
         filename = os.path.join(HERE, 'testfiles', 'overlays_bug_%d.xml' % number)
-        o = DbBase([filename], config)
+        o = DbBase(config, [filename])
         for verbose in (True, False):
             for t in o.list(verbose=verbose):
-                print t[0]
-                print
+                print(t[0].decode('utf-8'))
+                print()
 
     def test_184449(self):
         self._overlays_bug(184449)
@@ -43,17 +49,19 @@ class Unicode(unittest.TestCase):
         self._overlays_bug(286290)
 
 
-class FormatSubpathCategory(unittest.TestCase):
+class FormatBranchCategory(unittest.TestCase):
     def _run(self, number):
-        config = {}
+        #config = {'output': Message()}
+        config = BareConfig()
+        # Discuss renaming files to "branch-%d.xml"
         filename1 = os.path.join(HERE, 'testfiles',
                 'subpath-%d.xml' % number)
 
         # Read, write, re-read, compare
-        os1 = DbBase([filename1], config)
-        filename2 = os.tmpnam()
+        os1 = DbBase(config, [filename1])
+        filename2 = tempfile.mkstemp()[1]
         os1.write(filename2)
-        os2 = DbBase([filename2], config)
+        os2 = DbBase(config, [filename2])
         os.unlink(filename2)
         self.assertTrue(os1 == os2)
 
@@ -95,16 +103,16 @@ class TarAddRemoveSync(unittest.TestCase):
 """ % {     'temp_tarball_url':urllib.pathname2url(temp_tarball_path),
             'repo_name':repo_name}
         (fd, temp_collection_path) = tempfile.mkstemp()
-        f = os.fdopen(fd, 'w')
-        f.write(xml_text)
-        f.close()
+        with os.fdopen(fd, 'w') as f:
+            f.write(xml_text)
 
         # Make playground directory
         temp_dir_path = tempfile.mkdtemp()
 
         # Make DB from it
-        config = {'tar_command':'/bin/tar'}
-        db = DbBase([temp_collection_path], config)
+        #config = {'output': Message(), 'tar_command':'/bin/tar'}
+        config = BareConfig()
+        db = DbBase(config, [temp_collection_path])
 
         specific_overlay_path = os.path.join(temp_dir_path, repo_name)
         o = db.select('tar-test-overlay')
